@@ -2,7 +2,8 @@ use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
 use crate::route::InjectiveRoute;
-use cosmwasm_std::{Addr, Coin, CosmosMsg, CustomMsg};
+use cosmwasm_std::{Addr, Coin, CosmosMsg, CustomMsg, Decimal256 as Decimal};
+
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 #[serde(rename_all = "snake_case")]
 pub struct InjectiveMsgWrapper {
@@ -10,9 +11,9 @@ pub struct InjectiveMsgWrapper {
     pub msg_data: InjectiveMsg,
 }
 
-impl Into<CosmosMsg<InjectiveMsgWrapper>> for InjectiveMsgWrapper {
-    fn into(self) -> CosmosMsg<InjectiveMsgWrapper> {
-        CosmosMsg::Custom(self)
+impl From<InjectiveMsgWrapper> for CosmosMsg<InjectiveMsgWrapper> {
+    fn from(s: InjectiveMsgWrapper) -> CosmosMsg<InjectiveMsgWrapper> {
+        CosmosMsg::Custom(s)
     }
 }
 
@@ -29,8 +30,8 @@ pub struct OrderData {
 pub struct OrderInfo {
     pub subaccount_id: String,
     pub fee_recipient: String,
-    pub price: String,
-    pub quantity: String,
+    pub price: Decimal,
+    pub quantity: Decimal,
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
@@ -46,10 +47,59 @@ pub struct DerivativeOrder {
     pub market_id: String,
     pub order_info: OrderInfo,
     pub order_type: i32,
-    pub margin: String,
+    pub margin: Decimal,
     pub trigger_price: Option<String>,
 }
 
+impl DerivativeOrder {
+    pub fn new(
+        price: Decimal,
+        quantity: Decimal,
+        margin: Decimal,
+        is_buy: bool,
+        market_id: &str,
+        subaccount_id: &str,
+        fee_recipient: &str,
+    ) -> DerivativeOrder {
+        DerivativeOrder {
+            market_id: market_id.to_string(),
+            order_info: OrderInfo {
+                subaccount_id: subaccount_id.to_string(),
+                fee_recipient: fee_recipient.to_string(),
+                price,
+                quantity,
+            },
+            order_type: if is_buy { 1 } else { 2 }, // TODO PO-orders
+            margin,
+            trigger_price: None,
+        }
+    }
+    pub fn is_reduce_only(&self) -> bool {
+        self.margin.is_zero()
+    }
+    pub fn get_price(&self) -> Decimal {
+        self.order_info.price
+    }
+    pub fn get_qty(&self) -> Decimal {
+        self.order_info.quantity
+    }
+    pub fn get_val(&self) -> Decimal {
+        self.get_price() * self.get_qty()
+    }
+    pub fn get_margin(&self) -> Decimal {
+        self.margin
+    }
+    pub fn non_reduce_only_is_invalid(&self) -> bool {
+        self.get_margin().is_zero() || self.get_price().is_zero() || self.get_qty().is_zero()
+    }
+    pub fn reduce_only_price_is_invalid(&self) -> bool {
+        self.get_price().is_zero()
+    }
+    pub fn reduce_only_qty_is_invalid(&self) -> bool {
+        self.get_qty().is_zero()
+    }
+}
+
 /// InjectiveMsg is an override of CosmosMsg::Custom to add support for Injective's custom message types
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 #[serde(rename_all = "snake_case")]
@@ -86,8 +136,8 @@ pub fn create_subaccount_transfer_msg(
         route: InjectiveRoute::Exchange,
         msg_data: InjectiveMsg::SubaccountTransfer {
             sender,
-            source_subaccount_id: source_subaccount_id.to_string(),
-            destination_subaccount_id: destination_subaccount_id.to_string(),
+            source_subaccount_id,
+            destination_subaccount_id,
             amount,
         },
     }
@@ -107,8 +157,8 @@ pub fn create_batch_update_orders_msg(
     InjectiveMsgWrapper {
         route: InjectiveRoute::Exchange,
         msg_data: InjectiveMsg::BatchUpdateOrders {
-            sender: sender.to_string(),
-            subaccount_id: subaccount_id.to_string(),
+            sender,
+            subaccount_id,
             spot_market_ids_to_cancel_all,
             derivative_market_ids_to_cancel_all,
             spot_orders_to_cancel,
@@ -123,10 +173,7 @@ pub fn create_batch_update_orders_msg(
 pub fn create_derivative_market_order_msg(sender: String, order: DerivativeOrder) -> CosmosMsg<InjectiveMsgWrapper> {
     InjectiveMsgWrapper {
         route: InjectiveRoute::Exchange,
-        msg_data: InjectiveMsg::CreateDerivativeMarketOrder {
-            sender: sender.to_string(),
-            order,
-        },
+        msg_data: InjectiveMsg::CreateDerivativeMarketOrder { sender, order },
     }
     .into()
 }
